
function support_dom()
{
    var dom = ( typeof(document.implementation) != "undefined" );
    var dom_event = ( typeof(document.addEventListener) != "undefined" || typeof(document.attachEvent) != "undefined" );
    
    return ( dom && dom_event );
}

function DOM_Event_Collection()
{
    this.addListener = function(elem, eventType, listener, useCapture) {
        if( elem.addEventListener ) // Standard
        {
            elem.addEventListener(eventType, listener, useCapture);
        }
        else
        {
            if( typeof(elem.listeners) == "undefined" )
            {
                elem.listeners = [];
            }
            
            if( !elem.listeners[eventType] || elem.listeners[eventType].length == 0 )
            {
                elem.listeners[eventType] = { capturePhase : [], bubblePhase : [] };
                elem["on" + eventType]    = function() { DOM_Event.launch(elem); }
            }
            
            var listeners = ( useCapture ) ? elem.listeners[eventType].capturePhase : elem.listeners[eventType].bubblePhase;
            
            listeners.remove(listener);
            listeners[listeners.length] = listener;
        }
    }
    
    this.removeListener = function(elem, eventType, listener, useCapture) {
        if( elem.removeEventListener ) // Standard
        {
            elem.removeEventListener(eventType, listener, useCapture);
        }
        else
        {
            if( elem.listeners && elem.listeners[eventType] )
            {
                var listeners = elem.listeners[eventType];
                if( useCapture )
                {
                    listeners.capturePhase.remove(listener);
                }
                else
                {
                    listeners.bubblePhase.remove(listener);
                }
                
                if( listeners.capturePhase.length == 0 && listeners.bubblePhase.length == 0 )
                {
                    elem.listeners[eventType] = [];
                    elem["on" + eventType]    = null;
                }
            }
        }
    }
    
    this.launch = function(elem) {
        var event = DOM_Event.normalize(window.event);
        
        if( event.target && event.target == elem )
        {
            var elemList = [];
            var tmp      = event.target;
            
            do
            {
                if( tmp.listeners && tmp.listeners[event.type] && tmp.listeners[event.type].capturePhase.length )
                {
                    elemList[elemList.length] = tmp;
                }
                
                if( tmp.nodeType == tmp.DOCUMENT_NODE )// on s'arrte au #document
                {
                    if( window.listeners && window.listeners[event.type] && window.listeners[event.type].capturePhase.length )
                    {
                        elemList[elemList.length] = window;
                    }
                    
                    break;
                }
                
                tmp = tmp.parentNode;
            }
            while( tmp );
            
            for( var i = (elemList.length - 1); i >= 0; i-- )
            {
                var tab = elemList[i].listeners[event.type];
                event.currentTarget = elemList[i];
                event.eventPhase    = 2;//( event.currentTarget == event.target ) ? 2 : 1;
                for( var j = 0, m = tab.capturePhase.length; j < m; j++ )
                {
                    elemList[i].listener = tab.capturePhase[j];
                    elemList[i].listener(event);
                }
                
                //
                // La recommandation indique que lorsque la propagation est interrompue, 
                // tous les guetteurs sur la cible en cours sont nanmoins appells, d'o 
                // la prsence de cette condition ici, et non dans la boucle de dernier niveau.
                //
                if( event.cancelBubble )
                {
                    return;
                }
            }
            
            tmp = elemList = null;
        }
        
        event.currentTarget = elem;
        event.eventPhase    = 2;// ( event.currentTarget == event.target ) ? 2 : 3;
        var tab = elem.listeners[event.type];
        for( var i = 0, m = tab.bubblePhase.length; i < m; i++ )
        {
            elem.listener = tab.bubblePhase[i];
            elem.listener(event);
        }
    }
    
    this.normalize = function(event) {
        if( typeof(event.correction) != "undefined" && event.correction == true )
        {
            return event;
        }
        
        //
        // Standard ?
        // - layerX/layerY
        // - pageX/pageY
        // - keyCode
        // - charCode
        //
        var newEvent = {
            //
            // Miscellanious properties
            //
            type          : event.type,
            eventPhase    : ( typeof(event.eventPhase) != 'undefined' ) ? event.eventPhase : 0,
            bubbles       : ( typeof(event.bubbles) != 'undefined' ) ? event.bubbles : true,
            cancelable    : ( typeof(event.cancelable) != 'undefined' ) ? event.cancelable : true,
            timeStamp     : ( typeof(event.timeStamp) != 'undefined' ) ? event.timeStamp : 0,
            
            //
            // Targets
            //
            currentTarget : ( typeof(event.currentTarget) != 'undefined' ) ? event.currentTarget : null,
            target        : ( typeof(event.target) != 'undefined' ) ? event.target : event.srcElement,
            relatedTarget : ( typeof(event.relatedTarget) != 'undefined' ) ? event.relatedTarget : (( event.type == 'mouseover' ) ? event.fromElement : event.toElement),
            
            //
            // Mouse positions & key properties
            //
            layerX        : ( typeof(event.layerX) != 'undefined' ) ? event.layerX : event.offsetX,
            layerY        : ( typeof(event.layerY) != 'undefined' ) ? event.layerY : event.offsetY,
            pageX         : ( typeof(event.pageX) != 'undefined' ) ? event.pageX : 0,
            pageY         : ( typeof(event.pageY) != 'undefined' ) ? event.pageY : 0,
            clientX       : event.clientX,
            clientY       : event.clientY,
            screenX       : event.screenX,
            screenY       : event.screenY,
            myoffsetX     : 0,
            myoffsetY     : 0,
            
            keyCode       : ( typeof(event.keyCode) != 'undefined' ) ? event.keyCode : 0,
            altKey        : ( typeof(event.altKey) != 'undefined' ) ? event.altKey : false,
            ctrlKey       : ( typeof(event.ctrlKey) != 'undefined' ) ? event.ctrlKey : false,
            metaKey       : ( typeof(event.metaKey) != 'undefined' ) ? event.metaKey : false,
            shiftKey      : ( typeof(event.shiftKey) != 'undefined' ) ? event.shiftKey : false,
            button        : event.button,
            
            //
            // Internal property
            //
            correction    : false,
            
            //
            // Constants
            //
            CAPTURING_PHASE : 1,
            AT_TARGET       : 2,
            BUBBLING_PHASE  : 3,
            
            //
            // Methods
            //
            preventDefault : function() {
                if( typeof(event.preventDefault) != "undefined" )
                {
                    event.preventDefault();
                }
                else
                {
                    event.returnValue = false;
                }
            },
            
            stopPropagation : function() {
                if( typeof(event.stopPropagation) != "undefined" )
                {
                    event.stopPropagation();
                }
                else
                {
                    event.cancelBubble = true;
                }
            }
        };
        
        /*
         * Mozilla indique 2 pour les vnements capturs...
         *
        if( newEvent.eventPhase == 2 && event.currentTarget != event.target )
        {
            newEvent.eventPhase = 1;
        }*/
        
        if( typeof(event.timeStamp) == 'undefined' )
        {
            var thisTime = new Date();
            newEvent.timeStamp = Date.UTC(1970, 1, 1, thisTime.getHours(), thisTime.getMinutes(), thisTime.getSeconds()) + thisTime.getMilliseconds();
        }
        
        if( typeof(event.pageX) == 'undefined' )
        {
            if( document.documentElement )
            {
                newEvent.pageX = (event.clientX + document.documentElement.scrollLeft);
                newEvent.pageY = (event.clientY + document.documentElement.scrollTop);
            }
            else
            {
                newEvent.pageX = (event.clientX + document.body.scrollLeft);
                newEvent.pageY = (event.clientY + document.body.scrollTop);
            }
        }
        
        //
        // W3C:   0 - Left button, 1 - Middle button, 2 - Right button
        // MS:    1 - Left button, 4 - Middle button, 2 - Right button
        // Opera: 1 - Left button, 3 - Middle button, 2 - Right button
        //
        if( event.button != 0 && event.button != 2 && event.keyCode == 0 && ( typeof(event.which) == 'undefined' || event.which > 0 ) )
        {
            if( typeof(event.which) != 'undefined' )// Mozilla, Safari & Opera
            {
                newEvent.button = event.which;
            }
            
            newEvent.button--;
            
            if( newEvent.button == 2 || newEvent.button == 3 )// Correction Opera et MS
            {
                newEvent.button = 1;
            }
        }
        
        if( newEvent.bubbles == null )
        {
            // Chercher dans le tableau bubblesEvents
        }
        
        if( newEvent.cancelable == null )
        {
            // Chercher dans le tableau cancelableEvents
        }
        
        // TODO: view, detail
        
        newEvent.correction = true;// Afin de ne pas modifier l'objet une seconde fois
        
        return newEvent;
    }
    
    this.bubblesEvents    = ["click", "dblclick", "mousedown", "mouseup", "mouseover", "mouseout", "mousemove", "abort", "error", "select", "change", "submit", "reset", "resize", "scroll"];
    this.cancelableEvents = ["click", "dblclick", "mousedown", "mouseup", "mouseover", "mouseout", "submit"];
}

if( support_dom() )
{
    Array.prototype.remove = function(value) {
        for( var i = 0, m = this.length; i < m; i++ )
        {
            if( this[i] == value )
            {
                for( var j = i, n = (this.length - 1); j < n; j++ )
                {
                    this[j] = this[j+1];
                }
                
                this.length--;
            }
        }
    }
    
    if( typeof(window.getComputedStyle) == "undefined" )
	{
		window.getComputedStyle = function(elt, pseudoElt) {
			if( typeof(elt.currentStyle) != "undefined" )
			{// see also runtimeStyle
				return elt.currentStyle;
			}
			
			return null;
		}
	}
    
    var DOM_Event = new DOM_Event_Collection();
}

